@@ -1,7 +1,7 @@
 """
 
 @author: Edward R Jones
-@version 1.14
+@version 1.26
 @copyright 2020 - Edward R Jones, all rights reserved.
 """
 
@@ -33,22 +33,24 @@ class scrape(object):
         
     def newspaper_stories(words, urls=None, display=True):
         if urls == None:
-            news_urls = {'huffington': 'http://huffingtonpost.com', 
-                 'reuters': 'http://www.reuters.com', 
-                 'cbs-news': 'http://www.cbsnews.com',
-                 'usa-today': 'http://usatoday.com',
-                 'cnn': 'http://cnn.com',
-                 'npr': 'http://www.npr.org',
-                 'abc-news': 'http://abcnews.com',
-                 'us-news': 'http://www.usnews.com',
-                 'msn':  'http://msn.com',
-                 'pbs': 'http://www.pbs.org',
-                 'nbc-news':  'http://www.nbcnews.com',
-                 'fox': 'http://www.foxnews.com'}
+           news_urls = {
+                'huffington': 'http://huffingtonpost.com', 
+                'reuters':    'http://www.reuters.com', 
+                'cbs-news':   'http://www.cbsnews.com',
+                'usa-today':  'http://usatoday.com',
+                'cnn':        'http://cnn.com',
+                'npr':        'http://www.npr.org',
+                'abc-news':   'http://abcnews.com',
+                'us-news':    'http://www.usnews.com',
+                'msn':        'http://msn.com',
+                'pbs':        'http://www.pbs.org',
+                'nbc-news':   'http://www.nbcnews.com',
+                'fox':        'http://www.foxnews.com'}
         else:
             news_urls = urls
             
-        df_articles = pd.DataFrame(columns=['agency', 'url', 'story'])
+        df_articles = pd.DataFrame(columns=['agency', 'url', 'length', 'keywords', 
+                                        'title', 'summary', 'story'])
         n_articles  = {}
         today = str(date.today())
         for agency, url in news_urls.items():
@@ -64,12 +66,13 @@ class scrape(object):
                 for article in paper.articles:
                     # Exclude articles that are in a language other then en
                     # or contains mostly video or pictures
-                    if article.url.find('.video/')>=0 or \
-                       article.url.find('/video') >=0 or \
-                       article.url.find('/picture') >=0 or \
+                    if article.url.find('.video/')>=0    or \
+                       article.url.find('/video') >=0    or \
+                       article.url.find('/picture') >=0  or \
                        article.url.find('.pictures/')>=0 or \
-                       article.url.find('/photo') >=0 or \
-                       article.url.find('.photos/')>=0 or \
+                       article.url.find('/photo') >=0    or \
+                       article.url.find('.photos/')>=0   or \
+                       article.url.find('espanol')>=0    or \
                        article.url.find('.mx/' )>=0 or \
                        article.url.find('/mx.' )>=0 or \
                        article.url.find('.fr/' )>=0 or \
@@ -101,14 +104,15 @@ class scrape(object):
                                 article_collection.append(article.url)
                         continue
                     article_collection.append(article.url)
+            n_to_review = len(article_collection)
             if display:
-                print(len(article_collection), "Articles selected for download")
+                print("Number of articles reviewed from",agency,":", n_to_review)
             j = 0
             for article_url in article_collection:
                 j += 1
                 article = Article(article_url)
                 article.download()
-                m = article_url.find(".com")
+                m     = article_url.find(".com")
                 m_org = article_url.find(".org")
                 if m_org>m:
                     m = m_org
@@ -121,7 +125,7 @@ class scrape(object):
                 n = 0
                 # Allow for a maximum of 5 download failures
                 stop_sec=1 # Initial max wait time in seconds
-                while n<3:
+                while n<2:
                     try:
                         article.parse()
                         n = 99
@@ -140,21 +144,42 @@ class scrape(object):
                 if n != 99:
                     # download failed
                     continue
-                story          = article.text.lower()
-                url_lower_case = article.url.lower()
-                for word in words:
-                    flag = 0
-                    if url_lower_case.find(word)>0:
-                        flag = 1
-                        break
-                    if story.find(word)>0:
-                        flag = 1
-                        break
-                if flag == 0:
-                    continue
-                df_story    = pd.DataFrame([[agency, article_url, story]], \
-                                   columns=['agency', 'url', 'story'])
-                df_articles = df_articles.append(df_story)
+                article.nlp()
+                keywords = article.keywords
+                title    = article.title
+                summary  = article.summary
+                story    = article.text
+                # Verify the story contains at least one of the key words
+                found_it = False
+                story_lower_case = story.lower()
+                for i in range(len(words)):
+                    j = story_lower_case.find(words[i].lower())
+                    if j>= 0:
+                        found_it = True
+                        continue
+                if found_it:
+                    length   = len(story)
+                    df_story = pd.DataFrame([[agency, url, length, keywords,
+                                              title, summary, story]], 
+                                columns=['agency', 'url', 'length', 'keywords',
+                                             'title', 'summary', 'story'])
+                    # Check for an identical already in the file
+                    if df_articles.shape[0]==0:
+                        df_articles  = df_articles.append(df_story)
+                    else:
+                        # Verify this story is not already in df_articles
+                        same_story = False
+                        for i in range(df_articles.shape[0]):
+                            if title==df_articles['title'].iloc[i]:
+                                same_story   = True
+                                n_to_review -= 1
+                                continue
+                        if not(same_story):
+                            df_articles  = df_articles.append(df_story)
+                if display:
+                    print("Number of unique articles found in",agency,":", 
+                          n_to_review)
+            
                 n_articles_selected += 1
             n_articles[agency] = [n_articles_selected, len(article_collection)]
             if display:
@@ -202,64 +227,64 @@ class scrape(object):
     def newsapi_get_urls(search_words, key, urls=None):
         if urls==None:
             agency_urls = {
-                'abc-news': 'https://abcnews.go.com',
-                'al-jazeera-english': 'http://www.aljazeera.com',
-                'ars-technica': 'http://arstechnica.com',
-                'associated-press': 'https://apnews.com/',
-                'axios': 'https://www.axios.com',
-                'bleacher-report': 'http://www.bleacherreport.com',
-                'bloomberg': 'http://www.bloomberg.com',
-                'breitbart-news': 'http://www.breitbart.com',
-                'business-insider': 'http://www.businessinsider.com',
-                'buzzfeed': 'https://www.buzzfeed.com',
-                'cbs-news': 'http://www.cbsnews.com',
-                'cnbc': 'http://www.cnbc.com',
-                'cnn': 'http://us.cnn.com',
-                'crypto-coins-news': 'https://www.ccn.com',
-                'engadget': 'https://www.engadget.com',
-                'entertainment-weekly': 'http://www.ew.com',
-                'espn': 'http://espn.go.com',
-                'espn-cric-info': 'http://www.espncricinfo.com/',
-                'fortune': 'http://fortune.com',
-                'fox-news': 'http://www.foxnews.com',
-                'fox-sports': 'http://www.foxsports.com',
-                'google-news': 'https://news.google.com',
-                'hacker-news': 'https://news.ycombinator.com',
-                'ign': 'http://www.ign.com',
-                'mashable': 'http://mashable.com',
-                'medical-news-today': 'http://www.medicalnewstoday.com',
-                'msnbc': 'http://www.msnbc.com',
-                'mtv-news': 'http://www.mtv.com/news',
-                'national-geographic': 'http://news.nationalgeographic.com',
-                'national-review': 'https://www.nationalreview.com/',
-                'nbc-news': 'http://www.nbcnews.com',
-                'new-scientist': 'https://www.newscientist.com/section/news',
-                'newsweek': 'http://www.newsweek.com',
-                'new-york-magazine': 'http://nymag.com',
-                'next-big-future': 'https://www.nextbigfuture.com',
-                'nfl-news': 'http://www.nfl.com/news',
-                'nhl-news': 'https://www.nhl.com/news',
-                'politico': 'https://www.politico.com',
-                'polygon': 'http://www.polygon.com',
-                'recode': 'http://www.recode.net',
-                'reddit-r-all': 'https://www.reddit.com/r/all',
-                'reuters': 'http://www.reuters.com',
-                'techcrunch': 'https://techcrunch.com',
-                'techradar': 'http://www.techradar.com',
-                'the-american-conservative': 'http://www.theamericanconservative.com/',
-                'the-hill': 'http://thehill.com',
-                'the-huffington-post': 'http://www.huffingtonpost.com',
-                'the-new-york-times': 'http://www.nytimes.com',
-                'the-next-web': 'http://thenextweb.com',
-                'the-verge': 'http://www.theverge.com',
-                'the-wall-street-journal': 'http://www.wsj.com',
-                'the-washington-post': 'https://www.washingtonpost.com',
-                'the-washington-times': 'https://www.washingtontimes.com/',
-                'time': 'http://time.com',
-                'usa-today': 'http://www.usatoday.com/news',
-                'vice-news': 'https://news.vice.com',
-                'wired': 'https://www.wired.com'
-                    }
+            'abc-news': 'https://abcnews.go.com',
+            'al-jazeera-english': 'http://www.aljazeera.com',
+            'ars-technica': 'http://arstechnica.com',
+            'associated-press': 'https://apnews.com/',
+            'axios': 'https://www.axios.com',
+            'bleacher-report': 'http://www.bleacherreport.com',
+            'bloomberg': 'http://www.bloomberg.com',
+            'breitbart-news': 'http://www.breitbart.com',
+            'business-insider': 'http://www.businessinsider.com',
+            'buzzfeed': 'https://www.buzzfeed.com',
+            'cbs-news': 'http://www.cbsnews.com',
+            'cnbc': 'http://www.cnbc.com',
+            'cnn': 'http://us.cnn.com',
+            'crypto-coins-news': 'https://www.ccn.com',
+            'engadget': 'https://www.engadget.com',
+            'entertainment-weekly': 'http://www.ew.com',
+            'espn': 'http://espn.go.com',
+            'espn-cric-info': 'http://www.espncricinfo.com/',
+            'fortune': 'http://fortune.com',
+            'fox-news': 'http://www.foxnews.com',
+            'fox-sports': 'http://www.foxsports.com',
+            'google-news': 'https://news.google.com',
+            'hacker-news': 'https://news.ycombinator.com',
+            'ign': 'http://www.ign.com',
+            'mashable': 'http://mashable.com',
+            'medical-news-today': 'http://www.medicalnewstoday.com',
+            'msnbc': 'http://www.msnbc.com',
+            'mtv-news': 'http://www.mtv.com/news',
+            'national-geographic': 'http://news.nationalgeographic.com',
+            'national-review': 'https://www.nationalreview.com/',
+            'nbc-news': 'http://www.nbcnews.com',
+            'new-scientist': 'https://www.newscientist.com/section/news',
+            'newsweek': 'http://www.newsweek.com',
+            'new-york-magazine': 'http://nymag.com',
+            'next-big-future': 'https://www.nextbigfuture.com',
+            'nfl-news': 'http://www.nfl.com/news',
+            'nhl-news': 'https://www.nhl.com/news',
+            'politico': 'https://www.politico.com',
+            'polygon': 'http://www.polygon.com',
+            'recode': 'http://www.recode.net',
+            'reddit-r-all': 'https://www.reddit.com/r/all',
+            'reuters': 'http://www.reuters.com',
+            'techcrunch': 'https://techcrunch.com',
+            'techradar': 'http://www.techradar.com',
+            'the-american-conservative': 
+                        'http://www.theamericanconservative.com/',
+            'the-hill': 'http://thehill.com',
+            'the-huffington-post': 'http://www.huffingtonpost.com',
+            'the-new-york-times': 'http://www.nytimes.com',
+            'the-next-web': 'http://thenextweb.com',
+            'the-verge': 'http://www.theverge.com',
+            'the-wall-street-journal': 'http://www.wsj.com',
+            'the-washington-post': 'https://www.washingtonpost.com',
+            'the-washington-times': 'https://www.washingtontimes.com/',
+            'time': 'http://time.com',
+            'usa-today': 'http://www.usatoday.com/news',
+            'vice-news': 'https://news.vice.com',
+            'wired': 'https://www.wired.com'}
         else:
             agency_urls = urls
         if len(search_words)==0 or agency_urls==None:
